# Import classes and functions from RPE_Utils
from UtilsRPE.MainFunctions import parse_sources, process_sources, assign_sbits_to_rpe_variables, \
    create_read_precisions_module_and_add_to_file
from UtilsRPE.Functions import load_vault, load_list_of_files_to_keep_unmodified

if __name__ == "__main__":
    # Paths to source folders
    path_to_input_sources = "./ModelSources/Input/"
    path_to_preprocessed_sources = "./ModelSources/Processed/"
    path_to_output_sources = "./ModelSources/Output/"

    # Starting
    print("Starting the implementation of the reduced precision emulator in:\n   %s\n\n" % path_to_input_sources)

    # The vault is the database that will contain the information of the sources
    vault = None

    # Blacklist of files that should not be processed. Is read from file "list_of_files_to_keep_unmodified.txt"
    preprocess_black_list = load_list_of_files_to_keep_unmodified()

    # The code is first processed to replace real declarations with type(rpe_var)
    process_sources(path_to_input_sources, path_to_preprocessed_sources, vault, preprocess_blacklist=preprocess_black_list)

    # The code is parsed to obtain source information
    parse_sources(path_to_preprocessed_sources)

    # We load the code information into the vault
    vault = load_vault()

    # Process again the sources to fix call to functions, read/write, ...
    process_sources(path_to_preprocessed_sources, path_to_output_sources, vault, preprocess_blacklist=preprocess_black_list)

    # Insert precision specification
    assign_sbits_to_rpe_variables(path_to_output_sources, vault=vault)

    # Add the read_precisions module into a file (should be accessible for all the other modules)
    file_to_add_read_precisions_module = None
    create_read_precisions_module_and_add_to_file(path_to_output_sources, file_to_add_read_precisions_module, vault=vault)

